# @validate_dream
# def deep_dream_simple(
#     image_path=None,
#     img=None,
#     output_path="dream.png",
#     model_name="inception_v3",
#     learning_rate=0.01,
#     iterations=100,
#     save_video=False,
#     duration=3,
#     mirror_video=False,
#     config=None,
# ):
#     if image_path is None and img is None:
#         raise TypeError("Missing image_path or img argument")

#     base_image_path = Path(image_path)
#     output_path = Path(output_path)

#     base_model = tf.keras.applications.InceptionV3(
#         include_top=False, weights="imagenet"
#     )

#     names = ["mixed3", "mixed5"]
#     layers = [base_model.get_layer(name).output for name in names]

#     ft_ext = tf.keras.Model(inputs=base_model.input, outputs=layers)
#     dream_model = DeepDream(ft_ext)

#     img = get_image(base_image_path)
#     img = tf.keras.applications.inception_v3.preprocess_input(img)
#     img_shape = img.shape[1:3]

#     if config is None:
#         config = Config(
#             feature_extractor=dream_model,
#             layer_settings=dream_model.model.layers,
#             original_shape=img_shape,
#             save_video=save_video,
#             enable_framing=save_video,
#             max_frames_to_sample=iterations,
#         )

#     img = tf.keras.applications.inception_v3.preprocess_input(img)
#     img = tf.convert_to_tensor(img)

#     learning_rate = tf.convert_to_tensor(learning_rate)
#     iterations_remaining = iterations
#     iteration = 0
#     while iterations_remaining:
#         run_iterations = tf.constant(min(100, iterations_remaining))
#         iterations_remaining -= run_iterations
#         iteration += run_iterations

#         loss, img = dream_model.gradient_ascent_loop(
#             img, run_iterations, tf.constant(learning_rate), config
#         )

#         # display.clear_output(wait=True)
#         show(deprocess(img))
#         print("Iteration {}, loss {}".format(iteration, loss))

#     tf.keras.utils.save_img(output_path, img)
#     print(f"Dream image saved to {output_path}")

#     if save_video:
#         config.framer.to_video(output_path.stem + ".mp4", duration, mirror_video)

#     return img


# @validate_dream
# def deep_dream_octaved(
#     image_path,
#     output_path="dream.png",
#     model_name="inception_v3",
#     learning_rate=0.01,
#     iterations=100,
#     save_video=False,
#     duration=3,
#     mirror_video=False,
# ):
#     base_image_path = Path(image_path)
#     output_path = Path(output_path)

#     base_model = tf.keras.applications.InceptionV3(
#         include_top=False, weights="imagenet"
#     )

#     names = ["mixed3", "mixed5"]
#     layers = [base_model.get_layer(name).output for name in names]

#     ft_ext = tf.keras.Model(inputs=base_model.input, outputs=layers)
#     dream_model = DeepDream(ft_ext)

#     img = get_image(base_image_path)
#     img = tf.keras.applications.inception_v3.preprocess_input(img)
#     img_shape = img.shape[1:3]

#     config = Config(
#         feature_extractor=dream_model,
#         layer_settings=dream_model.model.layers,
#         original_shape=img_shape,
#         save_video=False,
#         enable_framing=save_video,
#         max_frames_to_sample=iterations * 5,  # 5 octaves
#     )

#     OCTAVE_SCALE = 1.30
#     img = tf.constant(np.array(img))
#     float_base_shape = tf.cast(tf.shape(img)[:-1], tf.float32)

#     for n in range(-2, 3):
#         if n == 2:
#             config.save_video = save_video

#         new_shape = tf.cast(float_base_shape * (OCTAVE_SCALE**n), tf.int32)
#         img = tf.image.resize(img, new_shape).numpy()
#         img = deep_dream_simple(
#             img=img,
#             model_name=model_name,
#             iterations=iterations,
#             learning_rate=learning_rate,
#             save_video=False,
#             duration=duration,
#             mirror_video=mirror_video,
#             config=config,
#         )

#     tf.keras.utils.save_img(output_path, img)
#     print(f"Dream image saved to {output_path}")

#     if save_video:
#         config.framer.to_video(output_path.stem + ".mp4", duration, mirror_video)

#     return img



from dreamify.utils.common import get_image, show


def test_get_image_by_URL():
    url = (
        "https://storage.googleapis.com/download.tensorflow.org/"
        "example_images/YellowLabradorLooking_new.jpg"
    )
    img = get_image(url, max_dim=500)
    show(img)

    img = get_image(url)
    show(img)


def test_get_image_by_path():
    path = "examples/example0.jpg"
    img = get_image(path, max_dim=500)
    show(img)

    img = get_image(path)
    show(img)



import os
import tempfile

import imageio.v2 as imageio
import numpy as np
import tensorflow as tf
from moviepy.video.fx import AccelDecel, TimeSymmetrize
from moviepy.video.VideoClip import DataVideoClip

from dreamify.utils.common import deprocess


class ImageToVideoConverter:
    def __init__(self, dimensions, max_frames_to_sample):
        self.dimensions = dimensions
        self.max_frames_to_sample = max_frames_to_sample
        self.curr_frame_idx = 0
        self.total_frames = 0
        self.num_frames_to_insert: int = 0
        self.FPS: int = 30

        self.temp_folder = tempfile.mkdtemp()
        print(f"Temporary folder created at {self.temp_folder}")

    def add_to_frames(self, frame):
        frame = tf.image.resize(frame, self.dimensions)
        frame = deprocess(frame).numpy().astype("float32")

        # Buffering the frame to the disk
        frame_filename = os.path.join(
            self.temp_folder, f"frame_{self.curr_frame_idx:04d}.png"
        )
        imageio.imwrite(
            frame_filename, frame.astype("uint8")
        )  # Save the frame as an image
        self.curr_frame_idx += 1

    def continue_framing(self):
        return self.curr_frame_idx < self.max_frames_to_sample  # Use total_frames here

    def to_video(
        self,
        output_path="dream.mp4",
        duration=3,
        extend_ending=False,
        mirror_video=False,
    ):
        self.duration = duration
        self.num_frames_to_insert = self.calculate_num_frames_to_insert()

        self.upsample()

        output_dir = os.path.dirname(output_path)
        if output_dir != "" and not os.path.exists(output_dir):
            os.makedirs(output_dir, exist_ok=True)
        output_path = os.path.join(output_dir, output_path)

        # Read buffered frames from disk
        frames = [
            imageio.imread(os.path.join(self.temp_folder, f"frame_{i:04d}.png"))
            for i in range(self.total_frames)  # Use total_frames here
        ]
        print(f"Number of images to frame: {len(frames)}")

        # Create the video
        vid = DataVideoClip(frames, lambda x: x, fps=self.FPS)
        if mirror_video:
            vid = TimeSymmetrize().apply(vid)
        vid = AccelDecel(new_duration=duration).apply(vid)
        vid.write_videofile(output_path)

        # Clean up ops
        self.cleanup_temp_folder()

    def upsample(self):
        new_frames = []

        # Upsample via frame-frame interpolation
        for i in range(self.curr_frame_idx - 1):
            frame1 = imageio.imread(
                os.path.join(self.temp_folder, f"frame_{i:04d}.png")
            )
            frame2 = imageio.imread(
                os.path.join(self.temp_folder, f"frame_{i + 1:04d}.png")
            )

            # Add original frame
            new_frames.append(frame1)
            self.total_frames += 1  # Update total frames count

            interpolated = self.interpolate_frames(
                frame1, frame2, self.num_frames_to_insert
            )
            new_frames.extend(interpolated)

        self.save_upsampled_frames(new_frames)

    def interpolate_frames(self, frame1, frame2, num_frames):
        alphas = np.linspace(0.0, 1.0, num_frames + 2)[1:-1]  # Avoid frames 0 and 1

        interpolated_frames = (1 - alphas[:, None, None, None]) * frame1 + alphas[
            :, None, None, None
        ] * frame2
        return interpolated_frames.astype("uint8")

    def save_upsampled_frames(self, new_frames):
        for idx, frame in enumerate(new_frames):
            frame_filename = os.path.join(
                self.temp_folder, f"frame_{self.total_frames:04d}.png"
            )
            imageio.imwrite(frame_filename, frame)
            self.total_frames += 1  # Update total frames count

    def cleanup_temp_folder(self):
        # Delete all frames in the temporary folder
        for file_name in os.listdir(self.temp_folder):
            file_path = os.path.join(self.temp_folder, file_name)
            if os.path.isfile(file_path):
                os.remove(file_path)
        os.rmdir(self.temp_folder)  # Remove the folder itself
        print(f"Temporary folder at {self.temp_folder} has been cleaned up")

    def calculate_num_frames_to_insert(self):
        """
        Calculate the number of frames to interpolate to ensure video smoothness of 30fps

        Derivation:
                30 = (max_frames_to_sample * num_frames_to_insert) // duration
             => 30 * duration = max_frames_to_sample * num_frames_to_insert
             => 30 * duration // max_frames_to_sample = num_frames_to_insert
             ≡ num_frames_to_insert = (30 * duration) // max_frames_to_sample
        """
        return (self.FPS * self.duration) // self.max_frames_to_sample

    def __hash__(self):
        return hash(self.name)



import functools


def validate_positive(value, name, allow_zero=True, value_type=(int, float)):
    """Helper function to validate that a value is positive (or non-negative if allow_zero=True)."""
    if value is None:
        return

    if isinstance(value, slice):
        if not all(
            isinstance(v, (int, type(None)))
            for v in (value.start, value.stop, value.step)
        ):
            raise ValueError(f"{name} must be a slice with integer (or None) values.")
        return

    if not isinstance(value, value_type) or (value < 0 if allow_zero else value <= 0):
        raise ValueError(
            f"{name} must be a {'non-negative' if allow_zero else 'positive'} {value_type[0].__name__}."
        )


def validate_dream(func):
    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        validate_positive(kwargs.get("learning_rate"), "learning_rate", allow_zero=True)
        validate_positive(
            kwargs.get("num_octave"), "num_octave", allow_zero=False, value_type=(int,)
        )
        validate_positive(
            kwargs.get("octave_scale"),
            "octave_scale",
            allow_zero=False,
            value_type=(float,),
        )
        validate_positive(
            kwargs.get("iterations"), "iterations", allow_zero=False, value_type=(int,)
        )
        validate_positive(kwargs.get("max_loss"), "max_loss", allow_zero=False)
        validate_positive(kwargs.get("duration"), "duration", allow_zero=True)

        return func(*args, **kwargs)

    return wrapper


__all__ = [validate_dream]



# import IPython.display as display
from pathlib import Path

import tensorflow as tf

from dreamify.lib import TiledGradients, validate_dream
from dreamify.utils.common import deprocess, get_image, show
from dreamify.utils.configure import Config


@validate_dream
def deepdream(
    image_path,
    output_path="dream.png",
    iterations=100,
    learning_rate=0.01,
    octaves=range(-2, 3),
    octave_scale=1.3,
    save_video=False,
    duration=3,
    mirror_video=False,
):
    base_image_path = Path(image_path)
    output_path = Path(output_path)

    base_model = tf.keras.applications.InceptionV3(
        include_top=False, weights="imagenet"
    )

    names = ["mixed3", "mixed5"]
    layers = [base_model.get_layer(name).output for name in names]

    ft_ext = tf.keras.Model(inputs=base_model.input, outputs=layers)
    get_tiled_gradients = TiledGradients(ft_ext)

    dream_model = tf.keras.Model(inputs=base_model.input, outputs=layers)

    img = get_image(base_image_path)
    base_shape = tf.shape(img)
    img = tf.keras.applications.inception_v3.preprocess_input(img)
    img_shape = img.shape[:-1]

    config = Config(
        feature_extractor=dream_model,
        layer_settings=layers,
        original_shape=img_shape,
        save_video=save_video,
        enable_framing=True,
        max_frames_to_sample=iterations * len(octaves),
    )

    for octave in octaves:
        new_size = tf.cast(tf.convert_to_tensor(base_shape[:-1]), tf.float32) * (
            octave_scale**octave
        )
        new_size = tf.cast(new_size, tf.int32)
        img = tf.image.resize(img, tf.cast(new_size, tf.int32))

        for iteration in range(iterations):
            gradients = get_tiled_gradients(img, new_size)
            img = img + gradients * learning_rate
            img = tf.clip_by_value(img, -1, 1)

            if iteration % 10 == 0:
                # display.clear_output(wait=True)
                show(deprocess(img))
                print("Octave {}, Iteration {}".format(octave, iteration))

            if config.enable_framing and config.framer.continue_framing():
                config.framer.add_to_frames(img)

    tf.keras.utils.save_img(output_path, img)
    print(f"Dream image saved to {output_path}")

    if save_video:
        config.framer.to_video(output_path.stem + ".mp4", duration, mirror_video)

    return img


def main(save_video=False, duration=3, mirror_video=False):
    url = (
        "https://storage.googleapis.com/download.tensorflow.org/"
        "example_images/YellowLabradorLooking_new.jpg"
    )

    deepdream(
        image_path=url,
        save_video=save_video,
    )


if __name__ == "__main__":
    main()



import tensorflow as tf

from dreamify.utils.deep_dream_utils.utils import calc_loss


def random_roll(img, maxroll):
    # Randomly shift the image to avoid tiled boundaries.
    shift = tf.random.uniform(
        shape=[2], minval=-maxroll, maxval=maxroll, dtype=tf.int32
    )
    img_rolled = tf.roll(img, shift=shift, axis=[0, 1])
    return shift, img_rolled


class TiledGradients(tf.Module):
    def __init__(self, model):
        self.model = model

    @tf.function(
        input_signature=(
            tf.TensorSpec(shape=[None, None, 3], dtype=tf.float32),
            tf.TensorSpec(shape=[2], dtype=tf.int32),
            tf.TensorSpec(shape=[], dtype=tf.int32),
        )
    )
    def __call__(self, img, img_size, tile_size=512):
        shift, img_rolled = random_roll(img, tile_size)

        # Initialize the image gradients to zero.
        gradients = tf.zeros_like(img_rolled)

        # Skip the last tile, unless there's only one tile.
        xs = tf.range(0, img_size[1], tile_size)[:-1]
        if not tf.cast(len(xs), bool):
            xs = tf.constant([0])
        ys = tf.range(0, img_size[0], tile_size)[:-1]
        if not tf.cast(len(ys), bool):
            ys = tf.constant([0])

        for x in xs:
            for y in ys:
                # Calculate the gradients for this tile.
                with tf.GradientTape() as tape:
                    # This needs gradients relative to `img_rolled`.
                    # `GradientTape` only watches `tf.Variable`s by default.
                    tape.watch(img_rolled)

                    # Extract a tile out of the image.
                    img_tile = img_rolled[y : y + tile_size, x : x + tile_size]
                    loss = calc_loss(img_tile, self.model)

                # Update the image gradients for this tile.
                gradients = gradients + tape.gradient(loss, img_rolled)

        # Undo the random shift applied to the image and its gradients.
        gradients = tf.roll(gradients, shift=-shift, axis=[0, 1])

        # Normalize the gradients.
        gradients /= tf.math.reduce_std(gradients) + 1e-8

        return gradients


__all__ = [TiledGradients]



import numpy as np
import PIL.Image
import tensorflow as tf
from IPython import display


def show(img):
    """Display an image."""
    img = np.array(img)
    img = np.squeeze(img)
    display.display(PIL.Image.fromarray(img))


def deprocess(img):
    """Normalize image for display."""
    img = tf.squeeze(img)
    img = 255 * (img + 1.0) / 2.0
    return tf.cast(img, tf.uint8)


def get_image(source, max_dim=None):
    """Retrieve an image from a URL or a local path and load it as a NumPy array."""

    source = str(source)

    if source.startswith("http"):  # A URL to some image
        name = source.split("/")[-1]
        image_path = tf.keras.utils.get_file(name, origin=source)
        img = PIL.Image.open(image_path)
    else:  # A directory path to some image
        img = tf.keras.utils.load_img(source)

    if max_dim:
        img.thumbnail((max_dim, max_dim))

    img = np.array(img)
    return img


def preprocess_image(image_path):
    img = tf.keras.utils.load_img(image_path)
    img = tf.keras.utils.img_to_array(img)
    img = tf.np.expand_dims(img, axis=0)
    img = tf.keras.applications.inception_v3.preprocess_input(img)
    return img



import warnings
from pathlib import Path

import numpy as np
import tensorflow as tf
from tensorflow import keras

from dreamify.lib import FeatureExtractor, validate_dream
from dreamify.utils.common import deprocess, get_image, show
from dreamify.utils.configure import Config
from dreamify.utils.dream_utils import gradient_ascent_loop

# from dreamify.utils.compare import main

warnings.filterwarnings(
    "ignore", category=UserWarning, module="keras.src.models.functional"
)


@validate_dream
def dream(
    image_path,
    output_path="dream.png",
    model_name="inception_v3",
    learning_rate=20.0,
    iterations=30,
    octaves=3,
    octave_scale=1.4,
    max_loss=15.0,
    save_video=False,
    duration=3,
    mirror_video=False,
):
    base_image_path = Path(image_path)
    output_path = Path(output_path)

    ft_ext = FeatureExtractor(model_name)

    original_img = get_image(base_image_path)
    original_img = np.expand_dims(original_img, axis=0)
    original_img = keras.applications.inception_v3.preprocess_input(original_img)
    original_shape = original_img.shape[1:3]

    config = Config(
        feature_extractor=ft_ext,
        layer_settings=ft_ext.layer_settings,
        original_shape=original_shape,
        save_video=save_video,
        enable_framing=save_video,
        max_frames_to_sample=iterations,
    )

    successive_shapes = [original_shape]
    for i in range(1, octaves):
        shape = tuple([int(dim / (octave_scale**i)) for dim in original_shape])
        successive_shapes.append(shape)
    successive_shapes = successive_shapes[::-1]

    shrunk_original_img = tf.image.resize(original_img, successive_shapes[0])

    img = tf.identity(original_img)
    for i, shape in enumerate(successive_shapes):
        print(
            f"\n\n{'_'*20} Processing octave {i + 1} with shape {successive_shapes[i]} {'_'*20}\n\n"
        )
        img = tf.image.resize(img, successive_shapes[i])
        img = gradient_ascent_loop(
            img,
            iterations=iterations,
            learning_rate=learning_rate,
            max_loss=max_loss,
            config=config,
        )
        upscaled_shrunk_original_img = tf.image.resize(
            shrunk_original_img, successive_shapes[i]
        )
        same_size_original = tf.image.resize(original_img, successive_shapes[i])
        lost_detail = same_size_original - upscaled_shrunk_original_img
        img += lost_detail
        shrunk_original_img = tf.image.resize(original_img, successive_shapes[i])

    img = deprocess(img)
    keras.utils.save_img(output_path, img)
    print(f"Dream image saved to {output_path}")

    show(img)

    if save_video:
        config.framer.to_video(output_path.stem + ".mp4", duration, mirror_video)


def main():
    dream("examples/example0.jpg", octaves=2, iterations=10)


# Compares all models and layer settings on an image
if __name__ == "__main__":
    # main()  # current implementation of comparison,py has circular import
    pass



from dreamify.lib.deep_dream import DeepDream
from dreamify.lib.feature_extractor import FeatureExtractor
from dreamify.lib.image_to_video_converter import ImageToVideoConverter
from dreamify.lib.tiled_gradients import TiledGradients
from dreamify.lib.validators import validate_dream

__all__ = [
    "DeepDream",
    "FeatureExtractor",
    "ImageToVideoConverter",
    "TiledGradients",
    "validate_dream",
]



import numpy as np
import tensorflow as tf
from tensorflow import keras
from tqdm import trange


def preprocess_image(image_path):
    img = keras.utils.load_img(image_path)
    img = keras.utils.img_to_array(img)
    img = np.expand_dims(img, axis=0)
    img = keras.applications.inception_v3.preprocess_input(img)
    return img


def compute_loss(input_image, config):
    features = config.feature_extractor(input_image)
    loss = tf.zeros(shape=())
    for name in features.keys():
        coeff = config.layer_settings[name]
        activation = features[name]
        loss += coeff * tf.reduce_mean(tf.square(activation[:, 2:-2, 2:-2, :]))
    return loss


@tf.function
def gradient_ascent_step(image, learning_rate, config):
    with tf.GradientTape() as tape:
        tape.watch(image)
        loss = compute_loss(image, config)
    grads = tape.gradient(loss, image)
    grads = tf.math.l2_normalize(grads)
    image += learning_rate * grads
    image = tf.clip_by_value(image, -1, 1)
    return loss, image


def gradient_ascent_loop(image, iterations, learning_rate, max_loss=None, config=None):
    for i in trange(
        iterations, desc="Gradient Ascent", unit="step", ncols=75, mininterval=0.1
    ):
        loss, image = gradient_ascent_step(image, learning_rate, config)

        if max_loss is not None and loss > max_loss:
            print(
                f"\nTerminating early: Loss ({loss:.2f}) exceeded max_loss ({max_loss:.2f}).\n"
            )
            break

        framer = config.framer

        if config.enable_framing and framer.continue_framing():
            framer.add_to_frames(image)

    return image


__all__ = [
    preprocess_image,
    gradient_ascent_loop,
]



# import os
# import tempfile

# import tensorflow as tf
# from moviepy import VideoFileClip
# from moviepy.video.compositing import CompositeVideoClip
# from moviepy.video.fx import AccelDecel, TimeSymmetrize
# from moviepy.video.VideoClip import DataVideoClip

# from dreamify.utils.common import deprocess


# class ImageToVideoConverterHybrid:
#     def __init__(self, dimensions, max_frames_to_sample, duration):
#         self.dimensions = dimensions
#         self.current_chunk: list = []
#         self.max_frames_to_sample: int = max_frames_to_sample
#         self.curr_frame_idx: int = 0
#         self.num_frames_to_insert: int = 0
#         self.duration: int = duration

#         self.FPS: int = 30
#         self.MAX_FRAMES_IN_MEM: int = 50

#         self.chunk_files: list = []
#         self.temp_folder = tempfile.mkdtemp()
#         print(f"Temporary folder created at {self.temp_folder}")

#     def add_to_frames(self, frame):
#         frame = tf.image.resize(frame, self.dimensions)
#         frame = deprocess(frame)
#         self.current_chunk.append(frame.numpy())
#         self.curr_frame_idx += 1

#         if len(self.current_chunk) >= self.MAX_FRAMES_IN_MEM:
#             self.flush_chunk()

#     def continue_framing(self):
#         return self.curr_frame_idx < self.max_frames_to_sample - 1

#     def flush_chunk(self):
#         if not self.current_chunk:
#             return

#         chunk_frames = []
#         for i in range(len(self.current_chunk) - 1):
#             chunk_frames.append(self.current_chunk[i])
#             interpolated = self.interpolate_frames(
#                 tf.cast(self.current_chunk[i], tf.float32),
#                 tf.cast(self.current_chunk[i + 1], tf.float32),
#                 30,
#             )
#             chunk_frames.extend(interpolated)

#         chunk_frames.append(self.current_chunk[-1])

#         clip = DataVideoClip(self.current_chunk, lambda x: x, fps=self.FPS)
#         chunk_path = os.path.join(
#             self.temp_folder, f"chunk_{len(self.chunk_files)}.mp4"
#         )
#         clip.write_videofile(chunk_path, logger=None)
#         self.chunk_files.append(chunk_path)
#         self.current_chunk = []

#     def to_video(
#         self,
#         output_path="dream.mp4",
#         duration=3,
#         mirror_video=False,
#     ):
#         self.duration = duration
#         self.flush_chunk()
#         clips = [VideoFileClip(chunk) for chunk in self.chunk_files]
#         final_clip = CompositeVideoClip.concatenate_videoclips(clips)
#         if mirror_video:
#             final_clip = TimeSymmetrize().apply(final_clip)
#         final_clip = AccelDecel(new_duration=duration).apply(final_clip)
#         final_clip.write_videofile(output_path, logger=None)

#     def interpolate_frames(self, frame1, frame2, num_frames):
#         alphas = tf.linspace(0.0, 1.0, num_frames + 2)[1:-1]
#         interpolated_frames = (1 - alphas[:, None, None, None]) * frame1 + alphas[
#             :, None, None, None
#         ] * frame2
#         return tf.cast(interpolated_frames, tf.uint8)

#     def calculate_num_frames_to_insert(self):
#         """
#         Calculate the number of frames to interpolate to ensure video smoothness of 30fps

#         Derivation:
#                 30 = (max_frames_to_sample * num_frames_to_insert) // duration
#              => 30 * duration = max_frames_to_sample * num_frames_to_insert
#              => 30 * duration // max_frames_to_sample = num_frames_to_insert
#              ≡ num_frames_to_insert = (30 * duration) // max_frames_to_sample
#         """
#         return (self.FPS * self.duration) // self.max_frames_to_sample



from pathlib import Path

import pytest

from dreamify.deepdream import deepdream


@pytest.fixture
def deepdream_fixture(request):
    iterations = getattr(request, "param", 100)

    url = (
        "https://storage.googleapis.com/download.tensorflow.org/"
        "example_images/YellowLabradorLooking_new.jpg"
    )

    return url, iterations


@pytest.mark.parametrize("deepdream_fixture", [1], indirect=True)
def test_mock_deepdream(deepdream_fixture):
    img_src, iterations = deepdream_fixture

    # Rolled
    deepdream(
        image_path=img_src,
        iterations=iterations,
        learning_rate=0.1,
        save_video=True,
        output_path="mock.png",
    )
    Path("mock.png").unlink(missing_ok=True)
    Path("mock.mp4").unlink(missing_ok=True)



from pathlib import Path

import pytest

from dreamify.dream import dream


@pytest.mark.filterwarnings(
    "ignore:The structure of `inputs` doesn't match the expected structure"
)
def test_mock_dream():
    img_path = "examples/example0.jpg"
    out_path = "examples/mock_dream.jpg"

    dream(img_path, output_path=out_path, octaves=1, iterations=5, learning_rate=15.0)
    Path(out_path).unlink(missing_ok=True)


def test_dream_validator():
    img_path = "examples/example0.jpg"
    out_path = "examples/mock_dream.jpg"

    with pytest.raises(ValueError):
        dream(
            img_path,
            output_path=out_path,
            num_octave=-1,
            iterations=5,
            learning_rate=15.0,
        )
        print("Handled negative")
